@@ -4,22 +4,25 @@ class ActionItem(object):
     text = ""
     confirmation = ""
     icon = ""
-    multiple = False
+    multiple = True
     func = None
 
-    def __init__(self, name, text, confirmation, icon, multiple, func):
+    def __init__(self, name, text, confirmation, icon, multiple, single, func):
         self.name = name
-        self.text = text
+        self.text = text or name
         self.confirmation = confirmation
         self.icon = icon
         self.multiple = multiple
+        self.single = single
         self.func = func
 
     def __repr__(self):
-        return "Action name:%s; text:%s; func:%s;" % (self.name, self.text, self.func.__name__)
+        return "Action name:%s; text:%s; confirmation:%s; func:%s;" % \
+                (self.name, self.text, self.confirmation, self.func.__name__)
 
 
-def action(name, text, confirmation=None, icon = None, multiple=False):
+def action(name, text, confirmation=None, icon=None,
+            multiple=True, single=True):
     """
         Use this decorator to expose actions
 
@@ -32,9 +35,13 @@ def action(name, text, confirmation=None, icon = None, multiple=False):
             unconditionally.
         :param icon:
             Font Awesome icon name
+        :param multiple:
+            If true will display action on list view
+        :param single:
+            If true will display action on show view
     """
     def wrap(f):
-        f._action = (name, text, confirmation, icon, multiple)
+        f._action = (name, text, confirmation, icon, multiple, single)
         return f
 
     return wrap
