@@ -1,10 +1,7 @@
 import logging
-from flask import Blueprint, request, redirect, session, url_for, render_template
-from flask.globals import _app_ctx_stack, _request_ctx_stack
-from werkzeug.urls import url_parse
+from flask import Blueprint, session, flash, render_template, url_for
 from .forms import GeneralModelConverter
 from .widgets import FormWidget, ShowWidget, ListWidget, SearchWidget
-from .models.filters import FilterRelationOneToManyEqual
 from .actions import ActionItem
 from .urltools import *
 
@@ -13,8 +10,8 @@ log = logging.getLogger(__name__)
 
 def expose(url='/', methods=('GET',)):
     """
-        Use this decorator to expose views in your view classes.
-       
+        Use this decorator to expose views on your view classes.
+
         :param url:
             Relative URL for the view
         :param methods:
@@ -26,17 +23,30 @@ def expose(url='/', methods=('GET',)):
             f._urls = []
         f._urls.append((url, methods))
         return f
+    return wrap
 
+
+def expose_api(name='', url='', methods=('GET',), description=''):
+    def wrap(f):
+        api_name = name or f.__name__
+        api_url = url or "/api/{0}".format(name)
+        if not hasattr(f, '_urls'):
+            f._urls = []
+            f._extra = {}
+        f._urls.append((api_url, methods))
+        f._extra[api_name] = (api_url, f.__name__, description)
+        return f
     return wrap
 
 
 class BaseView(object):
     """
-        All views inherit from this class. it's constructor will register your exposed urls on flask as a Blueprint.
+        All views inherit from this class.
+        it's constructor will register your exposed urls on flask as a Blueprint.
 
         This class does not expose any urls, but provides a common base for all views.
-        
-        Extend this class if you want to expose methods for your own templates        
+
+        Extend this class if you want to expose methods for your own templates
     """
 
     appbuilder = None
@@ -47,7 +57,9 @@ class BaseView(object):
     """ Override this if you want to define your own relative url """
 
     template_folder = 'templates'
+    """ The template folder relative location """
     static_folder = 'static'
+    """  The static folder relative location """
     base_permissions = None
     """
         List with allowed base permission.
@@ -58,15 +70,17 @@ class BaseView(object):
     """
 
     default_view = 'list'
-
+    """ the default view for this BaseView, to be used with url_for (method name) """
     extra_args = None
     """ dictionary for injecting extra arguments into template """
-
+    _apis = None
 
     def __init__(self):
         """
             Initialization of base permissions
             based on exposed methods and actions
+
+            Initialization of extra args
         """
         if self.base_permissions is None:
             self.base_permissions = set()
@@ -77,13 +91,18 @@ class BaseView(object):
             self.base_permissions = list(self.base_permissions)
         if not self.extra_args:
             self.extra_args = dict()
+        self._apis = dict()
+        for attr_name in dir(self):
+            if hasattr(getattr(self, attr_name), '_extra'):
+                _extra = getattr(getattr(self, attr_name), '_extra')
+                for key in _extra: self._apis[key] = _extra[key]
 
     def create_blueprint(self, appbuilder,
                          endpoint=None,
                          static_folder=None):
         """
             Create Flask blueprint. You will generally not use it
-            
+
             :param appbuilder:
                the AppBuilder object
             :param endpoint:
@@ -111,14 +130,12 @@ class BaseView(object):
                                        url_prefix=self.route_base,
                                        template_folder=self.template_folder,
                                        static_folder=static_folder)
-
         self._register_urls()
         return self.blueprint
 
     def _register_urls(self):
         for attr_name in dir(self):
             attr = getattr(self, attr_name)
-
             if hasattr(attr, '_urls'):
                 for url, methods in attr._urls:
                     self.blueprint.add_url_rule(url,
@@ -126,15 +143,23 @@ class BaseView(object):
                                                 attr,
                                                 methods=methods)
 
-
     def render_template(self, template, **kwargs):
+        """
+            Use this method on your own endpoints, will pass the extra_args
+            to the templates.
+
+            :param template: The template relative path
+            :param kwargs: arguments to be passed to the template
+        """
+        kwargs['base_template'] = self.appbuilder.base_template
+        kwargs['appbuilder'] = self.appbuilder
         return render_template(template, **dict(list(kwargs.items()) + list(self.extra_args.items())))
 
     def _prettify_name(self, name):
         """
             Prettify pythonic variable name.
 
-            For example, 'hello_world' will be converted to 'Hello World'
+            For example, 'HelloWorld' will be converted to 'Hello World'
 
             :param name:
                 Name to prettify.
@@ -150,85 +175,195 @@ class BaseView(object):
             :param name:
                 Name to prettify.
         """
-        return name.replace('_', ' ').title()
-
+        return re.sub('[._]', ' ', name).title()
 
     def update_redirect(self):
         """
             Call it on your own endpoint's to update the back history navigation.
-            If you bypass it, the next submit our back will go over it.
+            If you bypass it, the next submit or back will go over it.
         """
         page_history = Stack(session.get('page_history', []))
         page_history.push(request.url)
         session['page_history'] = page_history.to_json()
-        
+
     def get_redirect(self):
         """
             Returns the previous url.
         """
-        index_url = url_for('%s.%s' % (self.appbuilder.indexview.endpoint, self.appbuilder.indexview.default_view))
+        index_url = self.appbuilder.get_url_for_index
         page_history = Stack(session.get('page_history', []))
 
         if page_history.pop() is None:
             return index_url
         session['page_history'] = page_history.to_json()
-        redir = page_history.pop() or index_url
-        return redir
+        url = page_history.pop() or index_url
+        return url
+
+    @classmethod
+    def get_default_url(cls, **kwargs):
+        """
+            Returns the url for this class default endpoint
+        """
+        return url_for(cls.__name__ + '.' + cls.default_view, **kwargs)
+
+    def get_uninit_inner_views(self):
+        """
+            Will return a list with views that need to be initialized.
+            Normally related_views from ModelView
+        """
+        return []
+
+    def get_init_inner_views(self, views):
+        """
+            Sets initialized inner views
+        """
+        pass
+
+
+class BaseFormView(BaseView):
+    """
+        Base class FormView's
+    """
+    form_template = 'appbuilder/general/model/edit.html'
+
+    edit_widget = FormWidget
+    """ Form widget to override """
+    form_title = ''
+    """ The form title to be displayed """
+    form_columns = None
+    """ The form columns to include, if empty will include all"""
+    form = None
+    """ The WTF form to render """
+    form_fieldsets = None
+    """ Form field sets """
+    default_view = 'this_form_get'
+    """ The form view default entry endpoint """
+
+    def _init_vars(self):
+        self.form_columns = self.form_columns or []
+        self.form_fieldsets = self.form_fieldsets or []
+        list_cols = [field.name for field in self.form.refresh()]
+        if self.form_fieldsets:
+            self.form_columns = []
+            for fieldset_item in self.form_fieldsets:
+                self.form_columns = self.form_columns + list(fieldset_item[1].get('fields'))
+        else:
+            if not self.form_columns:
+                self.form_columns = list_cols
+
+    def form_get(self, form):
+        """
+            Override this method to implement your form processing
+        """
+        pass
+
+    def form_post(self, form):
+        """
+            Override this method to implement your form processing
+            
+            :param form: WTForm form
+            
+            Return None or a flask response to render 
+            a custom template or redirect the user
+        """
+        pass
+
+    def _get_edit_widget(self, form=None, exclude_cols=None, widgets=None):
+        exclude_cols = exclude_cols or []
+        widgets = widgets or {}
+        widgets['edit'] = self.edit_widget(route_base=self.route_base,
+                                           form=form,
+                                           include_cols=self.form_columns,
+                                           exclude_cols=exclude_cols,
+                                           fieldsets=self.form_fieldsets
+        )
+        return widgets
+
 
 
 class BaseModelView(BaseView):
     """
         The base class of ModelView and ChartView, all properties are inherited
         Customize ModelView and ChartView overriding this properties
+
+        This class supports all the basics for query
     """
 
     datamodel = None
-    """ 
+    """
         Your sqla model you must initialize it like::
-        
+
             class MyView(ModelView):
-                datamodel = SQLAModel(MyTable, db.session)
+                datamodel = SQLAInterface(MyTable)
     """
 
     title = 'Title'
 
     search_columns = None
-    """ 
-        List with allowed search columns, if not provided all possible search columns will be used 
-        If you want to limit the search (*filter*) columns possibilities, define it with a list of column names from your model::
-        
-            class MyView(ModelView):
-                datamodel = SQLAModel(MyTable, db.session)
-                search_columns = ['name','address']
-             
     """
-    label_columns = None
-    """ 
-        Dictionary of labels for your columns, override this if you want diferent pretify labels 
-        
-        example (will just override the label for name column)::
-        
+        List with allowed search columns, if not provided all possible search columns will be used
+        If you want to limit the search (*filter*) columns possibilities, define it with a list of column names from your model::
+
             class MyView(ModelView):
-                datamodel = SQLAModel(MyTable, db.session)
+                datamodel = SQLAInterface(MyTable)
+                search_columns = ['name','address']
+
+    """
+    search_exclude_columns = None
+    """
+        List with columns to exclude from search. Search includes all possible columns by default
+    """
+    search_form_extra_fields = None
+    """
+        A dictionary containing column names and a WTForm
+        Form fields to be added to the Add form, these fields do not
+        exist on the model itself ex::
+
+        search_form_extra_fields = {'some_col':BooleanField('Some Col', default=False)}
+
+    """
+    search_form_query_rel_fields = None
+    """
+        Add Customized query for related fields on search form.
+        Assign a dictionary where the keys are the column names of
+        the related models to filter, the value for each key, is a list of lists with the
+        same format as base_filter
+        {'relation col name':[['Related model col',FilterClass,'Filter Value'],...],...}
+        Add a custom filter to form related fields::
+
+            class ContactModelView(ModelView):
+                datamodel = SQLAModel(Contact, db.session)
+                search_form_query_rel_fields = [('group':[['name',FilterStartsWith,'W']]}
+
+    """
+
+    label_columns = None
+    """
+        Dictionary of labels for your columns, override this if you want diferent pretify labels
+
+        example (will just override the label for name column)::
+
+            class MyView(ModelView):
+                datamodel = SQLAInterface(MyTable)
                 label_columns = {'name':'My Name Label Override'}
-        
+
     """
     search_form = None
     """ To implement your own add WTF form for Search """
     base_filters = None
-    """ 
+    """
         Filter the view use: [['column_name',BaseFilter,'value'],]
-    
+
         example::
-        
+
             def get_user():
                 return g.user
-        
+
             class MyView(ModelView):
-                datamodel = SQLAModel(MyTable, db.session)
+                datamodel = SQLAInterface(MyTable)
                 base_filters = [['created_by', FilterEqualFunction, get_user],
                                 ['name', FilterStartsWith, 'a']]
-    
+
     """
 
     base_order = None
@@ -236,10 +371,13 @@ class BaseModelView(BaseView):
         Use this property to set default ordering for lists ('col_name','asc|desc')::
 
             class MyView(ModelView):
-                datamodel = SQLAModel(MyTable, db.session)
+                datamodel = SQLAInterface(MyTable)
                 base_order = ('my_column_name','asc')
 
     """
+    
+    search_widget = SearchWidget
+    """ Search widget you can override with your own """
 
     _base_filters = None
     """ Internal base Filter from class Filters will always filter view """
@@ -250,33 +388,51 @@ class BaseModelView(BaseView):
         """
             Constructor
         """
+        datamodel = kwargs.get('datamodel', None)
+        if datamodel:
+            self.datamodel = datamodel
         self._init_properties()
         self._init_forms()
         self._init_titles()
         super(BaseModelView, self).__init__(**kwargs)
 
+    def _gen_labels_columns(self, list_columns):
+        """
+            Auto generates pretty label_columns from list of columns
+        """
+        for col in list_columns:
+            if not self.label_columns.get(col):
+                self.label_columns[col] = self._prettify_column(col)
+
     def _init_titles(self):
         pass
 
     def _init_properties(self):
         self.label_columns = self.label_columns or {}
         self.base_filters = self.base_filters or []
-        self._base_filters = self.datamodel.get_filters().add_filter_list(self.datamodel, self.base_filters)
+        self.search_exclude_columns = self.search_exclude_columns or []
+        self.search_columns = self.search_columns or []
+
+        self._base_filters = self.datamodel.get_filters().add_filter_list(self.base_filters)
         list_cols = self.datamodel.get_columns_list()
-        self.search_columns = self.search_columns or self.datamodel.get_search_columns_list()
-        for col in list_cols:
-            if not self.label_columns.get(col):
-                self.label_columns[col] = self._prettify_column(col)
+        search_columns = self.datamodel.get_search_columns_list()
+        if not self.search_columns:
+            self.search_columns = [x for x in search_columns if x not in self.search_exclude_columns]
+
+        self._gen_labels_columns(list_cols)
         self._filters = self.datamodel.get_filters(self.search_columns)
-        
 
     def _init_forms(self):
         conv = GeneralModelConverter(self.datamodel)
         if not self.search_form:
-            self.search_form = conv.create_form(self.label_columns, self.search_columns)
-
-
-    def _get_search_widget(self, form=None, exclude_cols=[], widgets={}):
+            self.search_form = conv.create_form(self.label_columns, 
+                                                self.search_columns,
+                                                extra_fields=self.search_form_extra_fields,
+                                                filter_rel_fields=self.search_form_query_rel_fields)
+        
+    def _get_search_widget(self, form=None, exclude_cols=None, widgets=None):
+        exclude_cols = exclude_cols or []
+        widgets = widgets or {}
         widgets['search'] = self.search_widget(route_base=self.route_base,
                                                form=form,
                                                include_cols=self.search_columns,
@@ -285,6 +441,16 @@ class BaseModelView(BaseView):
         )
         return widgets
 
+    def _label_columns_json(self):
+        """
+            Prepares dict with labels to be JSON serializable
+        """
+        ret = {}
+        for key, value in list(self.label_columns.items()):
+            ret[key] = str(value.encode('UTF-8'))
+        return ret
+
+
 
 class BaseCRUDView(BaseModelView):
     """
@@ -293,14 +459,14 @@ class BaseCRUDView(BaseModelView):
     """
 
     related_views = None
-    """ 
+    """
         List with ModelView classes
         Will be displayed related with this one using relationship sqlalchemy property::
 
             class MyView(ModelView):
                 datamodel = SQLAModel(Group, db.session)
                 related_views = [MyOtherRelatedView]
-                
+
     """
     _related_views = None
     """ internal list with ref to instantiated view classes """
@@ -333,20 +499,30 @@ class BaseCRUDView(BaseModelView):
         A list of columns (or model's methods) to be displayed on the edit form view.
         Use it to control the order of the display
     """
+    show_exclude_columns = None
+    """
+       A list of columns to exclude from the show view. By default all columns are included.
+    """
+    add_exclude_columns = None
+    """
+       A list of columns to exclude from the add form. By default all columns are included.
+    """
+    edit_exclude_columns = None
+    """
+       A list of columns to exclude from the edit form. By default all columns are included.
+    """
     order_columns = None
     """ Allowed order columns """
-
     page_size = 10
-    """ 
-        Use this property to change default page size 
     """
-
+        Use this property to change default page size
+    """
     show_fieldsets = None
-    """ 
+    """
         show fieldsets django style [(<'TITLE'|None>, {'fields':[<F1>,<F2>,...]}),....]
-        
+
         ::
-        
+
             class MyView(ModelView):
                 datamodel = SQLAModel(MyTable, db.session)
 
@@ -357,18 +533,18 @@ class BaseCRUDView(BaseModelView):
 
     """
     add_fieldsets = None
-    """ 
+    """
         add fieldsets django style (look at show_fieldsets for an example)
     """
     edit_fieldsets = None
-    """ 
+    """
         edit fieldsets django style (look at show_fieldsets for an example)
     """
 
     description_columns = None
-    """ 
+    """
         Dictionary with column descriptions that will be shown on the forms::
-        
+
             class MyView(ModelView):
                 datamodel = SQLAModel(MyTable, db.session)
 
@@ -376,65 +552,52 @@ class BaseCRUDView(BaseModelView):
     """
     validators_columns = None
     """ Dictionary to add your own validators for forms """
+
     add_form_extra_fields = None
-    """ Dictionary to add extra fields to the Add form using this property """
+    """
+        A dictionary containing column names and a WTForm
+        Form fields to be added to the Add form, these fields do not
+        exist on the model itself ex::
+
+        add_form_extra_fields = {'some_col':BooleanField('Some Col', default=False)}
+
+    """
     edit_form_extra_fields = None
-    """ Dictionary to Add extra fields to the Edit form using this property """
-    add_form_query_cascade = None
-    """
-        FUTURE FEATURE, Don't use it yet
-        
-        Implements query cascade related fields. Will user relate fields
-        with multiple values.
+    """ Dictionary to add extra fields to the Edit form using this property """
 
-        Grammar: [('Parent field','Child field', SQLAModel(ChildModel, Session),
-                [['child field rel to parent',Filter,'parent'],...]),...]
-
-            class ContactModelView(ModelView):
-                datamodel = SQLAModel(Contact, db.session)
-                add_form_query_cascade = [('model', 'manufacture'
-                        SQLAModel(Product, db.session),
-                        [['manufacture_id',FilterEqual,'id']]
-                        )]
-
-    """
-    edit_form_query_cascade = None
-    """
-        FUTURE FEATURE, Don't use it yet
-    """
     add_form_query_rel_fields = None
     """
-        Add Customized query for related fields on add form.
-        Assign a list of tuples like ('relation col name',SQLAModel,[['Related model col',FilterClass,'Filter Value'],...])
+        Add Customized query for related fields to add form.
+        Assign a dictionary where the keys are the column names of
+        the related models to filter, the value for each key, is a list of lists with the
+        same format as base_filter
+        {'relation col name':[['Related model col',FilterClass,'Filter Value'],...],...}
         Add a custom filter to form related fields::
 
             class ContactModelView(ModelView):
                 datamodel = SQLAModel(Contact, db.session)
-                add_form_query_rel_fields = [('group',
-                        SQLAModel(Group, db.session),
-                        [['name',FilterStartsWith,'W']]
-                        )]
+                add_form_query_rel_fields = [('group':[['name',FilterStartsWith,'W']]}
 
     """
     edit_form_query_rel_fields = None
     """
-        Add Customized query for related fields on edit form.
-        Assign a list of tuples like ('relation col name',SQLAModel,[['Related model col',FilterClass,'Filter Value'],...])
+        Add Customized query for related fields to edit form.
+        Assign a dictionary where the keys are the column names of
+        the related models to filter, the value for each key, is a list of lists with the
+        same format as base_filter
+        {'relation col name':[['Related model col',FilterClass,'Filter Value'],...],...}
         Add a custom filter to form related fields::
 
             class ContactModelView(ModelView):
                 datamodel = SQLAModel(Contact, db.session)
-                edit_form_query_rel_fields = [('group',
-                        SQLAModel(Group, db.session),
-                        [['name',FilterStartsWith,'W']]
-                        )]
+                add_form_query_rel_fields = [('group':[['name',FilterStartsWith,'W']]}
 
     """
 
     add_form = None
-    """ To implement your own assign WTF form for Add """
+    """ To implement your own, assign WTF form for Add """
     edit_form = None
-    """ To implement your own assign WTF form for Edit """
+    """ To implement your own, assign WTF form for Edit """
 
     list_template = 'appbuilder/general/model/list.html'
     """ Your own add jinja2 template for list """
@@ -453,8 +616,6 @@ class BaseCRUDView(BaseModelView):
     """ Add widget override """
     show_widget = ShowWidget
     """ Show widget override """
-    search_widget = SearchWidget
-    """ Search widget you can override with your own """
 
     actions = None
 
@@ -469,7 +630,6 @@ class BaseCRUDView(BaseModelView):
                 self.base_permissions.append(action.name)
                 self.actions[action.name] = action
 
-
     def _init_forms(self):
         """
             Init forms for Add and Edit
@@ -482,17 +642,14 @@ class BaseCRUDView(BaseModelView):
                                              self.description_columns,
                                              self.validators_columns,
                                              self.add_form_extra_fields,
-                                             self.add_form_query_rel_fields,
-                                             self.add_form_query_cascade)
+                                             self.add_form_query_rel_fields)
         if not self.edit_form:
             self.edit_form = conv.create_form(self.label_columns,
                                               self.edit_columns,
                                               self.description_columns,
                                               self.validators_columns,
                                               self.edit_form_extra_fields,
-                                              self.edit_form_query_rel_fields,
-                                              self.edit_form_query_cascade)
-
+                                              self.edit_form_query_rel_fields)
 
     def _init_titles(self):
         """
@@ -522,38 +679,40 @@ class BaseCRUDView(BaseModelView):
         self.validators_columns = self.validators_columns or {}
         self.add_form_extra_fields = self.add_form_extra_fields or {}
         self.edit_form_extra_fields = self.edit_form_extra_fields or {}
+        self.show_exclude_columns = self.show_exclude_columns or []
+        self.add_exclude_columns = self.add_exclude_columns or []
+        self.edit_exclude_columns = self.edit_exclude_columns or []
         # Generate base props
-        order_cols = self.datamodel.get_order_columns_list()
         list_cols = self.datamodel.get_user_columns_list()
         self.list_columns = self.list_columns or [list_cols[0]]
-        self.order_columns = self.order_columns or order_cols
+        self._gen_labels_columns(self.list_columns)
+        self.order_columns = self.order_columns or self.datamodel.get_order_columns_list(list_columns=self.list_columns)
         if self.show_fieldsets:
             self.show_columns = []
             for fieldset_item in self.show_fieldsets:
                 self.show_columns = self.show_columns + list(fieldset_item[1].get('fields'))
         else:
             if not self.show_columns:
-                self.show_columns = list_cols
+                self.show_columns = [x for x in list_cols if x not in self.show_exclude_columns]
         if self.add_fieldsets:
             self.add_columns = []
             for fieldset_item in self.add_fieldsets:
                 self.add_columns = self.add_columns + list(fieldset_item[1].get('fields'))
         else:
             if not self.add_columns:
-                self.add_columns = list_cols
+                self.add_columns = [x for x in list_cols if x not in self.add_exclude_columns]
         if self.edit_fieldsets:
             self.edit_columns = []
             for fieldset_item in self.edit_fieldsets:
                 self.edit_columns = self.edit_columns + list(fieldset_item[1].get('fields'))
         else:
             if not self.edit_columns:
-                self.edit_columns = list_cols
-
+                self.edit_columns = [x for x in list_cols if x not in self.edit_exclude_columns]
 
     """
     -----------------------------------------------------
             GET WIDGETS SECTION
-    -----------------------------------------------------        
+    -----------------------------------------------------
     """
 
     def _get_related_view_widget(self, item, related_view,
@@ -561,15 +720,23 @@ class BaseCRUDView(BaseModelView):
                                  page=None, page_size=None):
 
         fk = related_view.datamodel.get_related_fk(self.datamodel.obj)
-        filters = self.datamodel.get_filters()
-        filters.add_filter_related_view(fk, FilterRelationOneToManyEqual,
-                                                    related_view.datamodel, self.datamodel.get_pk_value(item))
+        filters = related_view.datamodel.get_filters()
+        # Check if it's a many to one model relation
+        if related_view.datamodel.is_relation_many_to_one(fk):
+            filters.add_filter_related_view(fk, self.datamodel.FilterRelationOneToManyEqual,
+                                        self.datamodel.get_pk_value(item))
+        # Check if it's a many to many model relation
+        elif related_view.datamodel.is_relation_many_to_many(fk):
+            filters.add_filter_related_view(fk, self.datamodel.FilterRelationManyToManyEqual,
+                                        self.datamodel.get_pk_value(item))
+        else:
+            log.error("Can't find relation on related view {0}".format(related_view.name))
+            return None
         return related_view._get_view_widget(filters=filters,
                                              order_column=order_column,
                                              order_direction=order_direction,
                                              page=page, page_size=page_size)
 
-
     def _get_related_views_widgets(self, item, orders=None,
                                    pages=None, page_sizes=None,
                                    widgets=None, **args):
@@ -627,11 +794,9 @@ class BaseCRUDView(BaseModelView):
                                            pks=pks,
                                            actions=actions,
                                            filters=filters,
-                                           modelview_name=self.__class__.__name__
-        )
+                                           modelview_name=self.__class__.__name__)
         return widgets
 
-
     def _get_show_widget(self, id, widgets=None, actions=None, show_fieldsets=None):
         widgets = widgets or {}
         actions = actions or self.actions
@@ -668,13 +833,22 @@ class BaseCRUDView(BaseModelView):
         )
         return widgets
 
+    def get_uninit_inner_views(self):
+        """
+            Will return a list with views that need to be initialized.
+            Normally related_views from ModelView
+        """
+        return self.related_views
+
+    def get_init_inner_views(self):
+        return self._related_views
+
 
     """
     -----------------------------------------------------
             CRUD functions behaviour
-    -----------------------------------------------------        
+    -----------------------------------------------------
     """
-
     def _list(self):
         """
             list function logic, override to implement different logic
@@ -712,7 +886,6 @@ class BaseCRUDView(BaseModelView):
         return self._get_related_views_widgets(item, orders=orders,
                                                pages=pages, page_sizes=page_sizes, widgets=widgets)
 
-
     def _add(self):
         """
             Add function logic, override to implement diferent logic
@@ -729,8 +902,9 @@ class BaseCRUDView(BaseModelView):
                 item = self.datamodel.obj()
                 form.populate_obj(item)
                 self.pre_add(item)
-                self.datamodel.add(item)
-                self.post_add(item)
+                if self.datamodel.add(item):
+                    self.post_add(item)
+                flash(*self.datamodel.message)
                 return None
             else:
                 is_valid_form = False
@@ -763,8 +937,9 @@ class BaseCRUDView(BaseModelView):
             if form.validate():
                 form.populate_obj(item)
                 self.pre_update(item)
-                self.datamodel.edit(item)
-                self.post_update(item)
+                if self.datamodel.edit(item):
+                    self.post_update(item)
+                flash(*self.datamodel.message)
                 return None
             else:
                 is_valid_form = False
@@ -778,7 +953,6 @@ class BaseCRUDView(BaseModelView):
             self.update_redirect()
         return widgets
 
-
     def _delete(self, pk):
         """
             Delete function logic, override to implement diferent logic
@@ -789,11 +963,11 @@ class BaseCRUDView(BaseModelView):
         """
         item = self.datamodel.get(pk)
         self.pre_delete(item)
-        self.datamodel.delete(item)
-        self.post_delete(item)
+        if self.datamodel.delete(item):
+            self.post_delete(item)
+        flash(*self.datamodel.message)
         self.update_redirect()
 
-
     """
     ------------------------------------------------
                 HELPER FUNCTIONS
@@ -810,7 +984,6 @@ class BaseCRUDView(BaseModelView):
             field = getattr(form, filter_key)
             field.data = rel_obj
 
-
     def pre_update(self, item):
         """
             Override this, will be called before update
