@@ -1,11 +1,9 @@
 import logging
-from flask import render_template
 from flask.ext.babelpkg import lazy_gettext
-from .widgets import ChartWidget, DirectChartWidget, MultipleChartWidget
+from .widgets import ChartWidget, DirectChartWidget
 from .jsontools import dict_to_json
 from ..widgets import SearchWidget
 from ..security.decorators import has_access
-from ..models.filters import Filters, FilterRelationOneToManyEqual
 from ..baseviews import BaseModelView, expose
 from ..urltools import *
 from ..models.group import GroupByProcessData, DirectProcessData
@@ -15,9 +13,9 @@ log = logging.getLogger(__name__)
 
 class BaseChartView(BaseModelView):
     """
-        This is the base class for all chart views. 
-        Use SimpleByChartView or GroupByChartView, override their properties and these
-        to customise your charts
+        This is the base class for all chart views.
+        Use DirectByChartView or GroupByChartView, override their properties and their base classes
+        (BaseView, BaseModelView, BaseChartView) to customise your charts
     """
 
     chart_template = 'appbuilder/general/charts/chart.html'
@@ -50,7 +48,7 @@ class BaseChartView(BaseModelView):
 
     def __init__(self, **kwargs):
         self._init_titles()
-        super(BaseModelView, self).__init__(**kwargs)
+        super(BaseChartView, self).__init__(**kwargs)
 
 
     def _init_titles(self):
@@ -58,7 +56,7 @@ class BaseChartView(BaseModelView):
 
     def _get_chart_widget(self, filters=None,
                           widgets=None, **args):
-        pass
+        raise NotImplementedError
 
     def _get_view_widget(self, **kwargs):
         """
@@ -68,6 +66,196 @@ class BaseChartView(BaseModelView):
         return self._get_chart_widget(**kwargs).get('chart')
 
 
+class GroupByChartView(BaseChartView):
+
+    definitions = []
+    """
+        These charts can display multiple series,
+        based on columns or methods defined on models.
+        You can display multiple charts on the same view.
+        This data can be grouped and aggregated has you like.
+
+        :label: (optional) String label to display on chart selection.
+        :group: String with the column name or method from model.
+        :formatter: (optional) function that formats the output of 'group' key
+        :series: A list of tuples with the aggregation function and the column name
+                to apply the aggregation
+
+        ::
+
+            [{
+                'label': 'String',
+                'group': '<COLNAME>'|'<FUNCNAME>'
+                'formatter: <FUNC>
+                'series': [(<AGGR FUNC>, <COLNAME>|'<FUNCNAME>'),...]
+                }
+            ]
+
+        example::
+
+            class CountryGroupByChartView(GroupByChartView):
+                datamodel = SQLAInterface(CountryStats)
+                chart_title = 'Statistics'
+
+            definitions = [
+                {
+                    'label': 'Country Stat',
+                    'group': 'country',
+                    'series': [(aggregate_avg, 'unemployed_perc'),
+                           (aggregate_avg, 'population'),
+                           (aggregate_avg, 'college_perc')
+                          ]
+                }
+            ]
+
+    """
+    chart_type = 'ColumnChart'
+    chart_template = 'appbuilder/general/charts/jsonchart.html'
+    chart_widget = DirectChartWidget
+    ProcessClass = GroupByProcessData
+
+    def __init__(self, **kwargs):
+        super(GroupByChartView, self).__init__(**kwargs)
+        for definition in self.definitions:
+            col = definition.get('group')
+            # Setup labels
+            try:
+                self.label_columns[col] = definition.get('label') or self.label_columns[col]
+            except Exception:
+                self.label_columns[col] = self._prettify_column(col)
+            if not definition.get('label'):
+                definition['label'] = self.label_columns[col]
+            # Setup Series
+            for serie in definition['series']:
+                if isinstance(serie, tuple):
+                    if hasattr(serie[0], '_label'):
+                        key = serie[0].__name__ + serie[1]
+                        self.label_columns[key] = \
+                            serie[0]._label + ' ' + self._prettify_column(serie[1])
+                else:
+                    self.label_columns[serie] = self._prettify_column(serie)
+
+
+
+    def get_group_by_class(self, definition):
+        """
+            intantiates the processing class (Direct or Grouped) and returns it.
+        """
+        group_by = definition['group']
+        series = definition['series']
+        if 'formatter' in definition:
+            formatter = {group_by: definition['formatter']}
+        else:
+            formatter = {}
+        return self.ProcessClass([group_by], series, formatter)
+
+
+    def _get_chart_widget(self, filters=None,
+                          order_column='',
+                          order_direction='',
+                          widgets=None,
+                          direct=None,
+                          height=None,
+                          definition='',
+                          **args):
+
+        height = height or self.height
+        widgets = widgets or dict()
+        joined_filters = filters.get_joined_filters(self._base_filters)
+        # check if order_column may be database ordered
+        if not self.datamodel.get_order_columns_list([order_column]):
+            order_column = ''
+            order_direction = ''
+        count, lst = self.datamodel.query(filters=joined_filters,
+                                          order_column=order_column,
+                                          order_direction=order_direction)
+        if not definition:
+            definition = self.definitions[0]
+        group = self.get_group_by_class(definition)
+        value_columns = group.to_json(group.apply(lst, sort=order_column == ''), self.label_columns)
+        widgets['chart'] = self.chart_widget(route_base=self.route_base,
+                                             chart_title=self.chart_title,
+                                             chart_type=self.chart_type,
+                                             chart_3d=self.chart_3d,
+                                             height=height,
+                                             value_columns=value_columns,
+                                             modelview_name=self.__class__.__name__,
+                                              **args)
+        return widgets
+
+    @expose('/chart/<group_by>')
+    @expose('/chart/')
+    @has_access
+    def chart(self, group_by=0):
+        group_by = int(group_by)
+        form = self.search_form.refresh()
+        get_filter_args(self._filters)
+        widgets = self._get_chart_widget(filters=self._filters,
+                                         definition=self.definitions[group_by],
+                                         order_column=self.definitions[group_by]['group'],
+                                         order_direction='asc')
+        widgets = self._get_search_widget(form=form, widgets=widgets)
+        self.update_redirect()
+        return self.render_template(self.chart_template, route_base=self.route_base,
+                               title=self.chart_title,
+                               label_columns=self.label_columns,
+                               definitions=self.definitions,
+                               group_by_label=self.group_by_label,
+                               height=self.height,
+                               widgets=widgets,
+                               appbuilder=self.appbuilder)
+
+
+class DirectByChartView(GroupByChartView):
+    """
+        Use this class to display charts with multiple series,
+        based on columns or methods defined on models.
+        You can display multiple charts on the same view.
+
+        Default routing point is '/chart'
+
+        Setup definitions property to configure the chart
+
+        :label: (optional) String label to display on chart selection.
+        :group: String with the column name or method from model.
+        :formatter: (optional) function that formats the output of 'group' key
+        :series: A list of tuples with the aggregation function and the column name
+                to apply the aggregation
+
+        The **definitions** property respects the following grammar::
+
+            definitions = [
+                    {
+                     'label': 'label for chart definition',
+                     'group': '<COLNAME>'|'<MODEL FUNCNAME>',
+                     'formatter': <FUNC FORMATTER FOR GROUP COL>,
+                     'series': ['<COLNAME>'|'<MODEL FUNCNAME>',...]
+                    }, ...
+                  ]
+
+        example::
+
+            class CountryDirectChartView(DirectByChartView):
+                datamodel = SQLAInterface(CountryStats)
+                chart_title = 'Direct Data Example'
+
+                definitions = [
+                    {
+                        'label': 'Unemployment',
+                        'group': 'stat_date',
+                        'series': ['unemployed_perc',
+                            'college_perc']
+                    }
+                ]
+
+    """
+    ProcessClass = DirectProcessData
+
+
+#-------------------------------------------------------
+# DEPRECATED SECTION
+#-------------------------------------------------------
+
 class BaseSimpleGroupByChartView(BaseChartView):
     group_by_columns = []
     """ A list of columns to be possibly grouped by, this list must be filled """
@@ -76,7 +264,7 @@ class BaseSimpleGroupByChartView(BaseChartView):
         if not self.group_by_columns:
             raise Exception('Base Chart View property <group_by_columns> must not be empty')
         else:
-            super(BaseChartView, self).__init__(**kwargs)
+            super(BaseSimpleGroupByChartView, self).__init__(**kwargs)
 
     def _get_chart_widget(self, filters=None,
                           order_column='',
@@ -97,7 +285,9 @@ class BaseSimpleGroupByChartView(BaseChartView):
                                              chart_type=self.chart_type,
                                              chart_3d=self.chart_3d,
                                              height=height,
-                                             value_columns=value_columns, **args)
+                                             value_columns=value_columns,
+                                             modelview_name = self.__class__.__name__,
+                                             **args)
         return widgets
 
 
@@ -113,7 +303,7 @@ class BaseSimpleDirectChartView(BaseChartView):
         if not self.direct_columns:
             raise Exception('Base Chart View property <direct_columns> must not be empty')
         else:
-            super(BaseChartView, self).__init__(**kwargs)
+            super(BaseSimpleDirectChartView, self).__init__(**kwargs)
 
 
     def get_group_by_columns(self):
@@ -145,190 +335,19 @@ class BaseSimpleDirectChartView(BaseChartView):
                                              chart_type=self.chart_type,
                                              chart_3d=self.chart_3d,
                                              height=height,
-                                             value_columns=value_columns, **args)
+                                             value_columns=value_columns,
+                                             modelview_name = self.__class__.__name__,
+                                             **args)
         return widgets
 
 
-class GroupByChartView(BaseChartView):
-
-    definitions = []
-    """
-        These charts can display multiple series,
-        based on columns or methods defined on models.
-        You can display multiple charts on the same view.
-        This data can be grouped and aggregated has you like.
-
-        :label: (optional) String label to display on chart selection.
-        :group: String with the column name or method from model.
-        :formatter: (optional) function that formats the output of 'group' key
-        :series: A list of tuples with the aggregation function and the column name
-                to apply the aggregation
-
-        ::
-
-            [{
-                'label': 'String',
-                'group': '<COLNAME>'|'<FUNCNAME>'
-                'formatter: <FUNC>
-                'series': [(<AGGR FUNC>, <COLNAME>|'<FUNCNAME>'),...]
-                }
-            ]
-
-        example::
-
-            class CountryGroupByChartView(GroupByChartView):
-                datamodel = SQLAModel(CountryStats)
-                chart_title = 'Statistics'
-
-            definitions = [
-                {
-                    'label': 'Country Stat',
-                    'group': 'country',
-                    'series': [(aggregate_avg, 'unemployed_perc'),
-                           (aggregate_avg, 'population'),
-                           (aggregate_avg, 'college_perc')
-                          ]
-                }
-            ]
-
-    """
-    chart_type = 'ColumnChart'
-    chart_template = 'appbuilder/general/charts/jsonchart.html'
-    chart_widget = DirectChartWidget
-    ProcessClass = GroupByProcessData
-
-    def __init__(self, **kwargs):
-        super(BaseChartView, self).__init__(**kwargs)
-        for definition in self.definitions:
-            col = definition.get('group')
-            try:
-                self.label_columns[col] = definition.get('label') or self.label_columns[col]
-            except Exception:
-                self.label_columns[col] = self._prettify_column(col)
-            if not definition.get('label'):
-                definition['label'] = self.label_columns[col]
-            for serie in definition['series']:
-                if isinstance(serie, tuple):
-                    if hasattr(serie[0], '_label'):
-                        key = serie[0].__name__ + serie[1]
-                        self.label_columns[key] = \
-                            serie[0]._label + ' ' + self._prettify_column(serie[1])
-                else:
-                    self.label_columns[serie] = self._prettify_column(serie)
-
-
-
-    def get_group_by_class(self, definition):
-        group_by = definition['group']
-        series = definition['series']
-        if 'formatter' in definition:
-            formatter = {group_by: definition['formatter']}
-        else:
-            formatter = {}
-        return self.ProcessClass([group_by], series, formatter)
-
-
-    def _get_chart_widget(self, filters=None,
-                          order_column='',
-                          order_direction='',
-                          widgets=None,
-                          direct=None,
-                          height=None,
-                          definition='',
-                          **args):
-
-        height = height or self.height
-        widgets = widgets or dict()
-        joined_filters = filters.get_joined_filters(self._base_filters)
-        count, lst = self.datamodel.query(filters=joined_filters,
-                                          order_column=order_column,
-                                          order_direction=order_direction)
-        if not definition:
-            definition = self.definitions[0]
-        group = self.get_group_by_class(definition)
-        value_columns = group.to_json(group.apply(lst), self.label_columns)
-        widgets['chart'] = self.chart_widget(route_base=self.route_base,
-                                             chart_title=self.chart_title,
-                                             chart_type=self.chart_type,
-                                             chart_3d=self.chart_3d,
-                                             height=height,
-                                             value_columns=value_columns, **args)
-        return widgets
-
-    @expose('/chart/<group_by>')
-    @expose('/chart/')
-    @has_access
-    def chart(self, group_by=0):
-        group_by = int(group_by)
-        form = self.search_form.refresh()
-        get_filter_args(self._filters)
-        widgets = self._get_chart_widget(filters=self._filters,
-                                         definition=self.definitions[group_by])
-        widgets = self._get_search_widget(form=form, widgets=widgets)
-
-        return self.render_template(self.chart_template, route_base=self.route_base,
-                               title=self.chart_title,
-                               label_columns=self.label_columns,
-                               definitions=self.definitions,
-                               group_by_label=self.group_by_label,
-                               height=self.height,
-                               widgets=widgets,
-                               appbuilder=self.appbuilder)
-
-
-class DirectByChartView(GroupByChartView):
-    """
-        Use this class to display charts with multiple series,
-        based on columns or methods defined on models.
-        You can display multiple charts on the same view.
-
-        Default routing point is '/chart'
-
-        Setup definitions property to configure the chart
-
-        :label: (optional) String label to display on chart selection.
-        :group: String with the column name or method from model.
-        :formatter: (optional) function that formats the output of 'group' key
-        :series: A list of tuples with the aggregation function and the column name
-                to apply the aggregation
-
-        The **definitions** property respects the following grammar:
-
-            definitions = [
-                    {
-                     'label': 'label for chart definition',
-                     'group': '<COLNAME>'|'<MODEL FUNCNAME>',
-                     'formatter': <FUNC FORMATTER FOR GROUP COL>,
-                     'series': ['<COLNAME>'|'<MODEL FUNCNAME>',...]
-                    }, ...
-                  ]
-
-        example::
-
-            class CountryDirectChartView(DirectByChartView):
-                datamodel = SQLAModel(CountryStats)
-                chart_title = 'Direct Data Example'
-
-                definitions = [
-                    {
-                        'label': 'Unemployment',
-                        'group': 'stat_date',
-                        'series': ['unemployed_perc',
-                            'college_perc']
-                    }
-                ]
-
-    """
-    ProcessClass = DirectProcessData
-
-
 class ChartView(BaseSimpleGroupByChartView):
     """
         **DEPRECATED**
 
         Provides a simple (and hopefully nice) way to draw charts on your application.
 
-        This will show Google Charts based on group by of your tables.                
+        This will show Google Charts based on group by of your tables.
     """
 
     @expose('/chart/<group_by>')
@@ -389,7 +408,9 @@ class TimeChartView(BaseSimpleGroupByChartView):
                                              chart_type=self.chart_type,
                                              chart_3d=self.chart_3d,
                                              height=height,
-                                             value_columns=value_columns, **args)
+                                             value_columns=value_columns,
+                                             modelview_name=self.__class__.__name__,
+                                             **args)
         return widgets
 
 
@@ -426,7 +447,7 @@ class DirectChartView(BaseSimpleDirectChartView):
         No group by is processed, example::
 
             class StatsChartView(DirectChartView):
-                datamodel = SQLAModel(Stats)
+                datamodel = SQLAInterface(Stats)
                 chart_title = lazy_gettext('Statistics')
                 direct_columns = {'Some Stats': ('X_col_1', 'stat_col_1', 'stat_col_2'),
                                   'Other Stats': ('X_col2', 'stat_col_3')}
