@@ -9,7 +9,7 @@ class MenuItem(object):
     baseview = None
     childs = []
 
-    def __init__(self, name, href="", icon="", label="", childs=[], baseview=None):
+    def __init__(self, name, href="", icon="", label="", childs=None, baseview=None):
         self.name = name
         self.href = href
         self.icon = icon
@@ -27,7 +27,10 @@ class MenuItem(object):
             else:
                 return url_for('%s.%s' % (self.baseview.endpoint, self.baseview.default_view))
         else:
-            return self.href
+            try:
+                return url_for(self.href)
+            except:
+                return self.href
 
     def __repr__(self):
         return self.name
@@ -35,16 +38,21 @@ class MenuItem(object):
 
 class Menu(object):
     menu = None
-    reverse = True
 
-    def __init__(self, reverse=True):
+    def __init__(self, reverse=True, extra_classes=""):
         self.menu = []
-        self.reverse = reverse
+        if reverse:
+            extra_classes = extra_classes + "navbar-inverse"
+        self.extra_classes = extra_classes
+
+    @property
+    def reverse(self):
+        return "navbar-inverse" in self.extra_classes
 
     def get_list(self):
         return self.menu
 
-    def find(self, name, menu=[]):
+    def find(self, name, menu=None):
         """
             Finds a menu item by name and returns it.
 
@@ -68,23 +76,27 @@ class Menu(object):
         else:
             self.find(category).childs.append(MenuItem(name=category, icon=icon, label=label))
 
-
     def add_link(self, name, href="", icon="", label="", category="", category_icon="", category_label="",
                  baseview=None):
         label = label or name
         category_label = category_label or category
         if category == "":
-            self.menu.append(MenuItem(name=name, href=href, icon=icon, label=label, baseview=baseview))
+            self.menu.append(MenuItem(name=name,
+                href=href, icon=icon,
+                label=label, baseview=baseview))
         else:
             menu_item = self.find(category)
             if menu_item:
-                menu_item.childs.append(MenuItem(name=name, href=href, icon=icon, label=label, baseview=baseview))
+                new_menu_item = MenuItem(name=name,
+                                    href=href, icon=icon,
+                                    label=label, baseview=baseview)
+                menu_item.childs.append(new_menu_item)
             else:
                 self.add_category(category=category, icon=category_icon, label=category_label)
-                self.find(category).childs.append(MenuItem(name=name,
-                                                           href=href, icon=icon, label=label,
-                                                           baseview=baseview))
-
+                new_menu_item = MenuItem(name=name,
+                                    href=href, icon=icon, label=label,
+                                    baseview=baseview)
+                self.find(category).childs.append(new_menu_item)
 
     def add_separator(self, category=""):
         menu_item = self.find(category)
