@@ -1,78 +1,22 @@
+import datetime
 import logging
+from functools import reduce
 from flask_babelpkg import lazy_gettext
+from .filters import Filters
 
 log = logging.getLogger(__name__)
 
 
-class BaseFilter(object):
-    """
-        Base class for all data filters.
-        Sub class to im
-    """
-    column_name = ''
-    datamodel = None
-    model = None
-    name = ''
-    is_related_view = False
-    """
-        Sets this filter to a special kind for related views.
-        If true this filter was not set by the user
-    """
-
-    def __init__(self, column_name, datamodel, is_related_view=False):
-        """
-            Constructor.
-
-            :param column_name:
-                Model field name
-            :param datamodel:
-                The datamodel access class
-            :param is_related_view:
-                Optional internal parameter to filter related views
-        """
-        self.column_name = column_name
-        self.datamodel = datamodel
-        self.model = datamodel.obj
-        self.is_related_view = is_related_view
-
-    def apply(self, query, value):
-        """
-            Override this to implement you own new filters
-        """
-        pass
-
-    def __repr__(self):
-        return self.name
-
-
-class FilterRelation(BaseFilter):
-    """
-        Base class for all filters for relations
-    """
-    pass
-
-
-class BaseFilterConverter(object):
-    """
-        Base Filter Converter, all classes responsible
-        for the association of columns and possible filters
-        will inherite from this
-
-    """
-    def __init__(self, datamodel):
-        self.datamodel = datamodel
-
-    def convert(self, col_name):
-        for conversion in self.conversion_table:
-            if getattr(self.datamodel, conversion[0])(col_name):
-                return [item(col_name, self.datamodel) for item in conversion[1]]
-        log.warning('Filter type not supported for column: %s' % col_name)
-
-
-
 class BaseInterface(object):
+    """
+        Base class for all data model interfaces.
+        Sub class it to implement your own interface for some data engine.
+    """
     obj = None
 
+    filter_converter_class = None
+    """ when sub classing override with your own custom filter converter """
+
     """ Messages to display on CRUD Events """
     add_row_message = lazy_gettext('Added Row')
     edit_row_message = lazy_gettext('Changed Row')
@@ -82,10 +26,16 @@ class BaseInterface(object):
     edit_integrity_error_message = lazy_gettext('Integrity error, probably unique constraint')
     general_error_message = lazy_gettext('General Error')
 
+    """ Tuple with message and text with severity type ex: ("Added Row", "info") """
+    message = ()
+
     def __init__(self, obj):
         self.obj = obj
 
     def _get_attr_value(self, item, col):
+        if not hasattr(item, col):
+            # it's an inner obj attr
+            return reduce(getattr, col.split('.'), item)
         if hasattr(getattr(item, col), '__call__'):
             # its a function
             return getattr(item, col)()
@@ -93,10 +43,14 @@ class BaseInterface(object):
             # its attribute
             return getattr(item, col)
 
+    def get_filters(self, search_columns=None):
+        search_columns = search_columns or []
+        return Filters(self.filter_converter_class, self, search_columns)
+
     def get_values_item(self, item, show_columns):
         return [self._get_attr_value(item, col) for col in show_columns]
 
-    def get_values(self, lst, list_columns):
+    def _get_values(self, lst, list_columns):
         """
             Get Values: formats values for list template.
             returns [{'col_name':'col_value',....},{'col_name':'col_value',....}]
@@ -114,9 +68,40 @@ class BaseInterface(object):
             retlst.append(retdict)
         return retlst
 
+    def get_values(self, lst, list_columns):
+        """
+            Get Values: formats values for list template.
+            returns [{'col_name':'col_value',....},{'col_name':'col_value',....}]
+
+            :param lst:
+                The list of item objects from query
+            :param list_columns:
+                The list of columns to include
+        """
+        for item in lst:
+            retdict = {}
+            for col in list_columns:
+                retdict[col] = self._get_attr_value(item, col)
+            yield retdict
+
+    def get_values_json(self, lst, list_columns):
+        """
+            Converts list of objects from query to JSON
+        """
+        result = []
+        for item in self.get_values(lst, list_columns):
+            for key, value in list(item.items()):
+                if isinstance(value, datetime.datetime) or isinstance(value, datetime.date):
+                    value = value.isoformat()
+                    item[key] = value
+                if isinstance(value, list):
+                    item[key] = [str(v) for v in value]
+            result.append(item)
+        return result
+
     """
         Returns the models class name
-        usefull for auto title on views
+        useful for auto title on views
     """
     @property
     def model_name(self):
@@ -125,10 +110,6 @@ class BaseInterface(object):
     """
         Next methods must be overridden
     """
-    def get_filters(self, search_columns=None):
-        pass
-
-
     def query(self, filters=None, order_column='', order_direction='',
               page=None, page_size=None):
         pass
@@ -139,6 +120,12 @@ class BaseInterface(object):
     def is_file(self, col_name):
         return False
 
+    def is_gridfs_file(self, col_name):
+        return False
+
+    def is_gridfs_image(self, col_name):
+        return False
+
     def is_string(self, col_name):
         return False
 
@@ -148,6 +135,9 @@ class BaseInterface(object):
     def is_integer(self, col_name):
         return False
 
+    def is_numeric(self, col_name):
+        return False
+
     def is_float(self, col_name):
         return False
 
@@ -190,6 +180,12 @@ class BaseInterface(object):
     def is_fk(self, col_name):
         return False
 
+    def get_max_length(self, col_name):
+        return -1
+
+    def get_min_length(self, col_name):
+        return -1
+
     """
     -----------------------------------------
            FUNCTIONS FOR CRUD OPERATIONS
@@ -200,19 +196,19 @@ class BaseInterface(object):
         """
             Adds object
         """
-        pass
+        raise NotImplementedError
 
     def edit(self, item):
         """
             Edit (change) object
         """
-        pass
+        raise NotImplementedError
 
     def delete(self, item):
         """
             Deletes object
         """
-        pass
+        raise NotImplementedError
 
     def get_col_default(self, col_name):
         pass
@@ -228,7 +224,10 @@ class BaseInterface(object):
         """
             Returns the primary key name
         """
-        pass
+        raise NotImplementedError
+
+    def get_pk_value(self, item):
+        return getattr(item, self.get_pk_name())
 
     def get(self, pk):
         """
@@ -236,14 +235,24 @@ class BaseInterface(object):
         """
         pass
 
-    def get_model_relation(self, prop):
-        pass
+    def get_related_model(self, prop):
+        raise NotImplementedError
+
+    def get_related_interface(self, col_name):
+        """
+            Returns a BaseInterface for the related model
+            of column name.
+
+            :param col_name: Column name with relation
+            :return: BaseInterface
+        """
+        raise NotImplementedError
 
     def get_related_obj(self, col_name, value):
-        pass
+        raise NotImplementedError
 
     def get_related_fk(self, model):
-        pass
+        raise NotImplementedError
 
     def get_columns_list(self):
         """
@@ -253,18 +262,17 @@ class BaseInterface(object):
 
     def get_user_columns_list(self):
         """
-            Returns a list user viewable columns names
+            Returns a list of user viewable columns names
         """
         return self.get_columns_list()
 
-
     def get_search_columns_list(self):
         """
             Returns a list of searchable columns names
         """
         return []
 
-    def get_order_columns_list(self):
+    def get_order_columns_list(self, list_columns=None):
         """
             Returns a list of order columns names
         """
