@@ -2,13 +2,12 @@ __author__ = 'dpgaspar'
 
 import operator
 import os
-
+from ..._compat import with_metaclass
 
 #--------------------------------------
 #        Exceptions
 #--------------------------------------
 class PKMissingException(Exception):
-
     def __init__(self, model_name=''):
         message = 'Please set one primary key on: {0}'.format(model_name)
         super(PKMissingException, self).__init__(self, message)
@@ -30,7 +29,6 @@ class GenericColumn(object):
         return isinstance(value, self.col_type)
 
 
-
 class MetaGenericModel(type):
     """
         Meta class for GenericModel
@@ -56,7 +54,7 @@ class MetaGenericModel(type):
                 vol_col = dct[prop]
                 obj._col_defs[prop] = vol_col
         obj.properties = obj._col_defs
-        obj.columns = obj._col_defs.keys()
+        obj.columns = list(obj._col_defs.keys())
         for col in obj.columns:
             if obj._col_defs[col].primary_key:
                 obj.pk = col
@@ -64,7 +62,7 @@ class MetaGenericModel(type):
         return obj
 
 
-class GenericModel(object):
+class GenericModel(with_metaclass(MetaGenericModel, object)):
     """
         Generic Model class to define generic purpose models to use
         with the framework.
@@ -80,7 +78,6 @@ class GenericModel(object):
                 name = GenericColumn(str)
 
     """
-    __metaclass__ = MetaGenericModel
 
     def __init__(self, **kwargs):
         if not self.pk:
@@ -110,7 +107,15 @@ class GenericModel(object):
 
 
 class GenericSession(object):
+    """
+        This class is a base, you should subclass it
+        to implement your own generic data source.
 
+        Override at least the **all** method.
+
+        **GenericSession** will implement filter and orders
+        based on your data generation on the **all** method.
+    """
     def __init__(self):
         self._order_by_cmd = None
         self._filters_cmd = list()
@@ -133,6 +138,10 @@ class GenericSession(object):
         self.store[model_cls._name] = []
 
     def get(self, pk):
+        """
+            Returns the object for the key
+            Override it for efficiency.
+        """
         for item in self.store.get(self.query_class):
             # coverts pk value to correct type
             pk = item.properties[item.pk].col_type(pk)
@@ -140,6 +149,9 @@ class GenericSession(object):
                 return item
 
     def query(self, model_cls):
+        """
+            SQLAlchemy query like method
+        """
         self._filters_cmd = list()
         self.query_filters = list()
         self._order_by_cmd = None
@@ -164,12 +176,85 @@ class GenericSession(object):
     #           FUNCTIONS for FILTERS
     #-----------------------------------------
 
+    def starts_with(self, col_name, value):
+        self._filters_cmd.append((self._starts_with, col_name, value))
+        return self
+
+    def _starts_with(self, item, col_name, value):
+        lw_col = getattr(item, col_name)
+        try:
+            lw_col = lw_col.lower()
+        except:
+            return None
+        lw_value = value.lower()
+        lw_value_list = lw_value.split(' ')
+
+        for lw_item in lw_value_list:
+            if not lw_col.startswith(lw_item):
+                return None
+
+        return col_name
+
+    def greater(self, col_name, value):
+        self._filters_cmd.append((self._greater, col_name, value))
+        return self
+
+    def _greater(self, item, col_name, value):
+        if getattr(item, col_name) == None:
+            return False
+        else:
+            try:
+                right_val = float(value)
+                return getattr(item, col_name) > right_val
+            except:
+                return False
+
+    def smaller(self, col_name, value):
+        self._filters_cmd.append((self._smaller, col_name, value))
+        return self
+
+    def _smaller(self, item, col_name, value):
+        if getattr(item, col_name) == None:
+            return False
+        else:
+            try:
+                right_val = float(value)
+                return getattr(item, col_name) < right_val
+            except:
+                return False
+
+    def ilike(self, col_name, value):
+        self._filters_cmd.append((self._ilike, col_name, value))
+        return self
+
+    def _ilike(self, item, col_name, value):
+        lw_col = getattr(item, col_name)
+        try:
+            lw_col = lw_col.lower()
+        except:
+            return None
+        lw_value = value.lower()
+        lw_value_list = lw_value.split(' ')
+
+        for lw_item in lw_value_list:
+            if not lw_item in lw_col:
+                return None
+
+        return col_name
+
     def like(self, col_name, value):
         self._filters_cmd.append((self._like, col_name, value))
         return self
 
     def _like(self, item, col_name, value):
-        return value in getattr(item, col_name)
+        lw_col = getattr(item, col_name)
+        lw_value_list = value.split(' ')
+
+        for lw_item in lw_value_list:
+            if not lw_item in lw_col:
+                return None
+
+        return col_name
 
     def not_like(self, col_name, value):
         self._filters_cmd.append((self._not_like, col_name, value))
@@ -184,7 +269,10 @@ class GenericSession(object):
 
     def _equal(self, item, col_name, value):
         source_value = getattr(item, col_name)
-        return source_value == type(source_value)(value)
+        try:
+            return source_value == type(source_value)(value)
+        except:
+            return 0.0
 
     def not_equal(self, col_name, value):
         self._filters_cmd.append((self._not_equal, col_name, value))
@@ -194,16 +282,19 @@ class GenericSession(object):
         source_value = getattr(item, col_name)
         return source_value != type(source_value)(value)
 
-
-    def offset(self, offset = 0):
+    def offset(self, offset=0):
         self._offset = offset
         return self
 
-    def limit(self, limit = 0):
+    def limit(self, limit=0):
         self._limit = limit
         return self
 
     def all(self):
+        """
+            SQLA like 'all' method, will populate all rows and apply all
+            filters and orders to it.
+        """
         items = list()
         if not self._filters_cmd:
             items = self.store.get(self.query_class)
@@ -225,13 +316,14 @@ class GenericSession(object):
 
     def add(self, model):
         model_cls_name = model._name
-        cls_list =  self.store.get(model_cls_name)
+        cls_list = self.store.get(model_cls_name)
         if not cls_list:
             self.store[model_cls_name] = []
         self.store[model_cls_name].append(model)
 
+
 #-------------------------------------
-#                EXP
+#   Example of an Generic Data Source
 #-------------------------------------
 class PSModel(GenericModel):
     UID = GenericColumn(str)
@@ -245,11 +337,11 @@ class PSModel(GenericModel):
 
 
 class PSSession(GenericSession):
-
     regexp = "(\w+) +(\w+) +(\w+) +(\w+) +(\w+:\w+|\w+) (\?|tty\w+) +(\w+:\w+:\w+) +(.+)\n"
 
     def add_object(self, line):
         import re
+
         group = re.findall(self.regexp, line)
         if group:
             model = PSModel()
