@@ -3,9 +3,9 @@ import datetime
 import calendar
 import logging
 from itertools import groupby
-from flask.ext.babelpkg import lazy_gettext as _
+from flask_babelpkg import lazy_gettext as _
 from flask_appbuilder._compat import as_unicode
-
+from .. import const as c
 
 log = logging.getLogger(__name__)
 
@@ -45,7 +45,11 @@ def aggregate_avg(items, col):
         Function to use on Group by Charts.
         accepts a list and returns the average of the list's items
     """
-    return aggregate_sum(items, col) / aggregate_count(items, col)
+    try:
+        return aggregate_sum(items, col) / aggregate_count(items, col)
+    except:
+        log.warning(c.LOGMSG_WAR_DBI_AVG_ZERODIV)
+        return 0.0
 
 
 class BaseGroupBy(object):
@@ -151,6 +155,14 @@ class GroupByDateMonth(BaseGroupBy):
 
 
 class BaseProcessData(object):
+    """
+        Base class to process data. It will group data by one or many columns or functions.
+        The aggregation is made by an already defined function, or by a custom function
+
+        :group_bys_cols: A list of columns or functions to group data.
+        :aggr_by_cols: A list of tuples [(<AGGR FUNC>,'<COLNAME>'),...].
+        :formatter_by_cols: A dict.
+    """
 
     group_bys_cols = None
     # ['<COLNAME>',<FUNC>, ....]
@@ -164,7 +176,6 @@ class BaseProcessData(object):
         self.aggr_by_cols = aggr_by_cols
         self.formatter_by_cols = formatter_by_cols
 
-
     def attrgetter(self, *items):
         if len(items) == 1:
             attr = items[0]
@@ -177,10 +188,14 @@ class BaseProcessData(object):
         return g
 
     def resolve_attr(self, obj, attr):
+        if not hasattr(obj, attr):
+            # it's an inner obj attr
+            return reduce(getattr, attr.split('.'), obj)
         if hasattr(getattr(obj, attr), '__call__'):
             # its a function
             return getattr(obj, attr)()
         else:
+            # it's an attribute
             return getattr(obj, attr)
 
     def format_columns(self, *values):
@@ -198,15 +213,52 @@ class BaseProcessData(object):
     def apply(self, data):
         pass
 
+    def to_dict(self, data):
+        ret = []
+        for item in data:
+            row = {}
+            if not isinstance(item[0], tuple):
+                row[self.group_bys_cols[0]] = str(item[0])
+            else:
+                for group_col_data, i in zip(item[0], enumerate(item[0])):
+                    row[self.group_bys_cols[i]] = str(group_col_data)
+            for col_data, i in zip(item[1:], enumerate(item[1:])):
+                log.debug("{0},{1}".format(col_data, i))
+                key = self.aggr_by_cols[i].__name__ + self.aggr_by_cols[i]
+                if isinstance(col_data, datetime.date):
+                    row[key] = str(col_data)
+                else:
+                    row[key] = col_data
+            ret.append(row)
+        return ret
+
+
+
     def to_json(self, data, labels=None):
+        """
+            Will return a dict with Google JSON structure for charts
+
+            The Google structure::
+
+                {
+                    cols: [{id:<COL_NAME>, label:<LABEL FOR COL>, type: <COL TYPE>}, ...]
+                    rows: [{c: [{v: <COL VALUE}, ...], ... ]
+                }
+
+            :param data:
+            :param labels: dict with labels to include on Google JSON strcut
+            :return: dict with Google JSON structure
+        """
         labels = labels or dict()
         json_data = dict()
         json_data['cols'] = []
+        # Create Structure to identify the grouped columns
         for group_col in self.group_bys_cols:
             label = '' or as_unicode(labels[group_col])
             json_data['cols'].append({'id': group_col,
                                       'label': label,
                                       'type': 'string'})
+        # Create Structure to identify the Aggregated columns
         for aggr_col in self.aggr_by_cols:
             if isinstance(aggr_col, tuple):
                 label_key = aggr_col[0].__name__ + aggr_col[1]
@@ -217,17 +269,18 @@ class BaseProcessData(object):
             json_data['cols'].append({'id': aggr_col,
                                       'label': label,
                                       'type': 'number'})
+        # Create Structure with the data
         json_data['rows'] = []
         for item in data:
             row = {'c': []}
             if not isinstance(item[0], tuple):
-                row['c'].append({'v': str(item[0])})
+                row['c'].append({'v': '{0}'.format(item[0])})
             else:
                 for group_col_data in item[0]:
-                    row['c'].append({'v': str(group_col_data)})
+                    row['c'].append({'v': '{0}'.format(group_col_data)})
             for col_data in item[1:]:
                 if isinstance(col_data, datetime.date):
-                    row['c'].append({'v': (str(col_data))})
+                    row['c'].append({'v': '{0}'.format(col_data)})
                 else:
                     row['c'].append({'v': col_data})
             json_data['rows'].append(row)
@@ -236,9 +289,10 @@ class BaseProcessData(object):
 
 class DirectProcessData(BaseProcessData):
 
-    def apply(self, data):
+    def apply(self, data, sort=True):
         group_by = self.group_bys_cols[0]
-        data = sorted(data, key=self.attrgetter(group_by))
+        if sort:
+            data = sorted(data, key=self.attrgetter(group_by))
         result = []
         for item in data:
             result_item = [self.format_columns(self.attrgetter(group_by)(item))]
@@ -249,9 +303,16 @@ class DirectProcessData(BaseProcessData):
 
 
 class GroupByProcessData(BaseProcessData):
+    """
+        Groups by data by chosen columns (property group_bys_cols).
 
-    def apply(self, data):
-        data = sorted(data, key=self.attrgetter(*self.group_bys_cols))
+        :data: A list of objects
+        :sort: boolean, if true python will sort the data
+        :return: A List of lists with group column and aggregation
+    """
+    def apply(self, data, sort=True):
+        if sort:
+            data = sorted(data, key=self.attrgetter(*self.group_bys_cols))
         result = []
         for (grouped, items) in groupby(data, key=self.attrgetter(*self.group_bys_cols)):
             items = list(items)
