@@ -1,10 +1,11 @@
 import datetime
 import logging
 
-from sqlalchemy import Column, Integer, ForeignKey, DateTime
-from sqlalchemy.orm import relationship
+from sqlalchemy import Column, Integer, ForeignKey, DateTime, String, Boolean
+from sqlalchemy.orm import relationship, backref, remote
 import sqlalchemy.types as types
 from sqlalchemy.ext.declarative import declared_attr
+from .._compat import as_unicode
 
 from flask import g
 
@@ -21,10 +22,16 @@ class FileColumn(types.TypeDecorator):
 
 class ImageColumn(types.TypeDecorator):
     """
-        Extends SQLAlchemy to support and mostly identify a Image Column
+        Extends SQLAlchemy to support and mostly identify an Image Column
+
     """
     impl = types.Text
 
+    def __init__(self, thumbnail_size=(20, 20, True), size=(100, 100, True), **kw):
+        types.TypeDecorator.__init__(self, **kw)
+        self.thumbnail_size = thumbnail_size
+        self.size = size
+        
 
 class AuditMixin(object):
     """
@@ -59,17 +66,24 @@ class AuditMixin(object):
     def changed_by(cls):
         return relationship("User", primaryjoin='%s.changed_by_fk == User.id' % cls.__name__)
 
-
     @classmethod
     def get_user_id(cls):
         try:
-            log.debug("GET USER ID: {0}".format(g.user.id))
             return g.user.id
         except Exception as e:
-            #log.warning("AuditMixin Get User ID {0}".format(str(e)))
+            # log.warning("AuditMixin Get User ID {0}".format(str(e)))
             return None
 
 
+class UserExtensionMixin(object):
+    __tablename__ = 'ab_user_extended'
+    __mapper_args__ = {'polymorphic_identity': 'ab_user_extended'}
+
+    @declared_attr
+    def id(cls):
+        return Column(None, ForeignKey('ab_user.id'), primary_key=True)
+
+
 """
     This is for retro compatibility
 """
