@@ -3,22 +3,28 @@ import sys
 import logging
 import sqlalchemy as sa
 
-from flask import flash
+from . import filters
 from sqlalchemy.orm import joinedload
 from sqlalchemy.exc import IntegrityError
 from sqlalchemy import func
 
 from ..base import BaseInterface
-from .filters import SQLAFilterConverter
-from ..filters import Filters
 from ..group import GroupByDateYear, GroupByDateMonth, GroupByCol
 from ..mixins import FileColumn, ImageColumn
 from ...filemanager import FileManager, ImageManager
 from ..._compat import as_unicode
+from ...const import LOGMSG_ERR_DBI_ADD_GENERIC, LOGMSG_ERR_DBI_EDIT_GENERIC, LOGMSG_ERR_DBI_DEL_GENERIC, \
+                     LOGMSG_WAR_DBI_ADD_INTEGRITY, LOGMSG_WAR_DBI_EDIT_INTEGRITY, LOGMSG_WAR_DBI_DEL_INTEGRITY
 
 log = logging.getLogger(__name__)
 
 
+def _include_filters(obj):
+    for key in filters.__all__:
+        if not hasattr(obj, key):
+            setattr(obj, key, getattr(filters, key))
+
+
 class SQLAInterface(BaseInterface):
     """
     SQLAModel
@@ -26,7 +32,10 @@ class SQLAInterface(BaseInterface):
     """
     session = None
 
+    filter_converter_class = filters.SQLAFilterConverter
+
     def __init__(self, obj, session=None):
+        _include_filters(self)
         self.list_columns = dict()
         self.list_properties = dict()
 
@@ -46,17 +55,18 @@ class SQLAInterface(BaseInterface):
         """
         return self.obj.__name__
 
-    @staticmethod
-    def _get_base_query(query=None, filters=None, order_column='', order_direction=''):
+    def _get_base_query(self, query=None, filters=None, order_column='', order_direction=''):
         if filters:
             query = filters.apply_all(query)
         if order_column != '':
+            # if Model has custom decorator **renders('<COL_NAME>')**
+            # this decorator will add a property to the method named *_col_name*
+            if hasattr(self.obj, order_column):
+                if hasattr(getattr(self.obj, order_column), '_col_name'):
+                    order_column = getattr(getattr(self.obj, order_column),'_col_name')
             query = query.order_by(order_column + ' ' + order_direction)
         return query
 
-    def get_filters(self, search_columns=[]):
-        return Filters(SQLAFilterConverter, search_columns, self)
-
     def query(self, filters=None, order_column='', order_direction='',
               page=None, page_size=None):
         """
@@ -74,11 +84,19 @@ class SQLAInterface(BaseInterface):
 
         """
         query = self.session.query(self.obj)
+        if len(order_column.split('.')) >= 2:
+            tmp_order_column = ''
+            for join_relation in order_column.split('.')[:-1]:
+                model_relation = self.get_related_model(join_relation)
+                query = query.join(model_relation)
+                # redefine order column name, because relationship can have a different name
+                # from the related table name.
+                tmp_order_column = tmp_order_column + model_relation.__tablename__ + '.'
+            order_column = tmp_order_column + order_column.split('.')[-1]
         query_count = self.session.query(func.count('*')).select_from(self.obj)
 
         query_count = self._get_base_query(query=query_count,
                                            filters=filters)
-
         query = self._get_base_query(query=query,
                                      filters=filters,
                                      order_column=order_column,
@@ -121,53 +139,103 @@ class SQLAInterface(BaseInterface):
     """
 
     def is_image(self, col_name):
-        return isinstance(self.list_columns[col_name].type, ImageColumn)
+        try:
+            return isinstance(self.list_columns[col_name].type, ImageColumn)
+        except:
+            return False
 
     def is_file(self, col_name):
-        return isinstance(self.list_columns[col_name].type, FileColumn)
+        try:
+            return isinstance(self.list_columns[col_name].type, FileColumn)
+        except:
+            return False
 
     def is_string(self, col_name):
-        return isinstance(self.list_columns[col_name].type, sa.types.String)
+        try:
+            return isinstance(self.list_columns[col_name].type, sa.types.String)
+        except:
+            return False
 
     def is_text(self, col_name):
-        return isinstance(self.list_columns[col_name].type, sa.types.Text)
+        try:
+            return isinstance(self.list_columns[col_name].type, sa.types.Text)
+        except:
+            return False
 
     def is_integer(self, col_name):
-        return isinstance(self.list_columns[col_name].type, sa.types.Integer)
+        try:
+            return isinstance(self.list_columns[col_name].type, sa.types.Integer)
+        except:
+            return False
+
+    def is_numeric(self, col_name):
+        try:
+            return isinstance(self.list_columns[col_name].type, sa.types.Numeric)
+        except:
+            return False
 
     def is_float(self, col_name):
-        return isinstance(self.list_columns[col_name].type, sa.types.Float)
+        try:
+            return isinstance(self.list_columns[col_name].type, sa.types.Float)
+        except:
+            return False
 
     def is_boolean(self, col_name):
-        return isinstance(self.list_columns[col_name].type, sa.types.Boolean)
+        try:
+            return isinstance(self.list_columns[col_name].type, sa.types.Boolean)
+        except:
+            return False
 
     def is_date(self, col_name):
-        return isinstance(self.list_columns[col_name].type, sa.types.Date)
+        try:
+            return isinstance(self.list_columns[col_name].type, sa.types.Date)
+        except:
+            return False
 
     def is_datetime(self, col_name):
-        return isinstance(self.list_columns[col_name].type, sa.types.DateTime)
-
+        try:
+            return isinstance(self.list_columns[col_name].type, sa.types.DateTime)
+        except:
+            return False
 
     def is_relation(self, col_name):
-        return isinstance(self.list_properties[col_name], sa.orm.properties.RelationshipProperty)
+        try:
+            return isinstance(self.list_properties[col_name], sa.orm.properties.RelationshipProperty)
+        except:
+            return False
 
     def is_relation_many_to_one(self, col_name):
-        if self.is_relation(col_name):
-            return self.list_properties[col_name].direction.name == 'MANYTOONE'
+        try:
+            if self.is_relation(col_name):
+                return self.list_properties[col_name].direction.name == 'MANYTOONE'
+        except:
+            return False
 
     def is_relation_many_to_many(self, col_name):
-        if self.is_relation(col_name):
-            return self.list_properties[col_name].direction.name == 'MANYTOMANY'
+        try:
+            if self.is_relation(col_name):
+                return self.list_properties[col_name].direction.name == 'MANYTOMANY'
+        except:
+            return False
 
     def is_relation_one_to_one(self, col_name):
-        if self.is_relation(col_name):
-            return self.list_properties[col_name].direction.name == 'ONETOONE'
+        try:
+            if self.is_relation(col_name):
+                return self.list_properties[col_name].direction.name == 'ONETOONE'
+        except:
+            return False
 
     def is_relation_one_to_many(self, col_name):
-        if self.is_relation(col_name):
-            return self.list_properties[col_name].direction.name == 'ONETOMANY'
+        try:
+            if self.is_relation(col_name):
+                return self.list_properties[col_name].direction.name == 'ONETOMANY'
+        except:
+            return False
 
     def is_nullable(self, col_name):
+        if self.is_relation_many_to_one(col_name):
+            col = self.get_relation_fk(col_name)
+            return col.nullable
         try:
             return self.list_columns[col_name].nullable
         except:
@@ -191,6 +259,17 @@ class SQLAInterface(BaseInterface):
         except:
             return False
 
+    def get_max_length(self, col_name):
+        try:
+            col = self.list_columns[col_name]
+            if col.type.length:
+                return col.type.length
+            else:
+                return -1
+        except:
+                return -1
+
+    
 
     """
     -----------------------------------------
@@ -202,16 +281,16 @@ class SQLAInterface(BaseInterface):
         try:
             self.session.add(item)
             self.session.commit()
-            flash(as_unicode(self.add_row_message), 'success')
+            self.message = (as_unicode(self.add_row_message), 'success')
             return True
         except IntegrityError as e:
-            flash(as_unicode(self.add_integrity_error_message), 'warning')
-            log.warning("Add record integrity error: {0}".format(str(e)))
+            self.message = (as_unicode(self.add_integrity_error_message), 'warning')
+            log.warning(LOGMSG_WAR_DBI_ADD_INTEGRITY.format(str(e)))
             self.session.rollback()
             return False
         except Exception as e:
-            flash(as_unicode(self.general_error_message + ' ' + str(sys.exc_info()[0])), 'danger')
-            log.exception("Add record error: {0}".format(str(e)))
+            self.message = (as_unicode(self.general_error_message + ' ' + str(sys.exc_info()[0])), 'danger')
+            log.exception(LOGMSG_ERR_DBI_ADD_GENERIC.format(str(e)))
             self.session.rollback()
             return False
 
@@ -219,38 +298,57 @@ class SQLAInterface(BaseInterface):
         try:
             self.session.merge(item)
             self.session.commit()
-            flash(as_unicode(self.edit_row_message), 'success')
+            self.message = (as_unicode(self.edit_row_message), 'success')
             return True
         except IntegrityError as e:
-            flash(as_unicode(self.edit_integrity_error_message), 'warning')
-            log.warning("Edit record integrity error: {0}".format(str(e)))
+            self.message = (as_unicode(self.edit_integrity_error_message), 'warning')
+            log.warning(LOGMSG_WAR_DBI_EDIT_INTEGRITY.format(str(e)))
             self.session.rollback()
             return False
         except Exception as e:
-            flash(as_unicode(self.general_error_message + ' ' + str(sys.exc_info()[0])), 'danger')
-            log.exception("Edit record error: {0}".format(str(e)))
+            self.message = (as_unicode(self.general_error_message + ' ' + str(sys.exc_info()[0])), 'danger')
+            log.exception(LOGMSG_ERR_DBI_EDIT_GENERIC.format(str(e)))
             self.session.rollback()
             return False
 
-
     def delete(self, item):
         try:
             self._delete_files(item)
             self.session.delete(item)
             self.session.commit()
-            flash(as_unicode(self.delete_row_message), 'success')
+            self.message = (as_unicode(self.delete_row_message), 'success')
             return True
         except IntegrityError as e:
-            flash(as_unicode(self.delete_integrity_error_message), 'warning')
-            log.warning("Delete record integrity error: {0}".format(str(e)))
+            self.message = (as_unicode(self.delete_integrity_error_message), 'warning')
+            log.warning(LOGMSG_WAR_DBI_DEL_INTEGRITY.format(str(e)))
             self.session.rollback()
             return False
         except Exception as e:
-            flash(as_unicode(self.general_error_message + ' ' + str(sys.exc_info()[0])), 'danger')
-            log.exception("Delete record error: {0}".format(str(e)))
+            self.message = (as_unicode(self.general_error_message + ' ' + str(sys.exc_info()[0])), 'danger')
+            log.exception(LOGMSG_ERR_DBI_DEL_GENERIC.format(str(e)))
             self.session.rollback()
             return False
 
+    def delete_all(self, items):
+        try:
+            for item in items:
+                self._delete_files(item)
+                self.session.delete(item)
+            self.session.commit()
+            self.message = (as_unicode(self.delete_row_message), 'success')
+            return True
+        except IntegrityError as e:
+            self.message = (as_unicode(self.delete_integrity_error_message), 'warning')
+            log.warning(LOGMSG_WAR_DBI_DEL_INTEGRITY.format(str(e)))
+            self.session.rollback()
+            return False
+        except Exception as e:
+            self.message = (as_unicode(self.general_error_message + ' ' + str(sys.exc_info()[0])), 'danger')
+            log.exception(LOGMSG_ERR_DBI_DEL_GENERIC.format(str(e)))
+            self.session.rollback()
+            return False
+
+
     """
     FILE HANDLING METHODS
     """
@@ -265,7 +363,6 @@ class SQLAInterface(BaseInterface):
             if self.is_image(file_col):
                 im.save_file(this_request.files[file_col], getattr(item, file_col))
 
-
     def _delete_files(self, item):
         for file_col in self.get_file_column_list():
             if self.is_file(file_col):
@@ -295,20 +392,18 @@ class SQLAInterface(BaseInterface):
                         return None
                 return value
 
-    def get_model_relation(self, col_name):
+    def get_related_model(self, col_name):
         return self.list_properties[col_name].mapper.class_
 
     def query_model_relation(self, col_name):
-        model = self.get_model_relation(col_name)
+        model = self.get_related_model(col_name)
         return self.session.query(model).all()
 
-    def _get_related_model(self, col_name):
-        if self.is_relation(col_name):
-            return self.get_model_relation(col_name), self.list_properties[col_name].direction.name
-        return None
+    def get_related_interface(self, col_name):
+        return self.__class__(self.get_related_model(col_name), self.session)
 
     def get_related_obj(self, col_name, value):
-        rel_model, rel_direction = self._get_related_model(col_name)
+        rel_model = self.get_related_model(col_name)
         return self.session.query(rel_model).get(value)
 
     def get_related_fks(self, related_views):
@@ -317,9 +412,8 @@ class SQLAInterface(BaseInterface):
     def get_related_fk(self, model):
         for col_name in self.list_properties.keys():
             if self.is_relation(col_name):
-                if model == self.get_model_relation(col_name):
+                if model == self.get_related_model(col_name):
                     return col_name
-                    #return self.get_property_col(i)
 
 
     """
@@ -337,7 +431,7 @@ class SQLAInterface(BaseInterface):
             Returns all model's columns except pk or fk
         """
         ret_lst = list()
-        for col_name in self.list_properties.keys():
+        for col_name in self.get_columns_list():
             if (not self.is_pk(col_name)) and (not self.is_fk(col_name)):
                 ret_lst.append(col_name)
         return ret_lst
@@ -345,7 +439,7 @@ class SQLAInterface(BaseInterface):
     #TODO get different solution, more integrated with filters
     def get_search_columns_list(self):
         ret_lst = list()
-        for col_name in self.list_properties.keys():
+        for col_name in self.get_columns_list():
             if not self.is_relation(col_name):
                 tmp_prop = self.get_property_first_col(col_name).name
                 if (not self.is_pk(tmp_prop)) and \
@@ -358,11 +452,23 @@ class SQLAInterface(BaseInterface):
                 ret_lst.append(col_name)
         return ret_lst
 
-    def get_order_columns_list(self):
+    def get_order_columns_list(self, list_columns=None):
+        """
+            Returns the columns that can be ordered
+
+            :param list_columns: optional list of columns name, if provided will
+                use this list only.
+        """
         ret_lst = list()
-        for col_name in self.list_properties.keys():
+        list_columns = list_columns or self.get_columns_list()
+        for col_name in list_columns:
             if not self.is_relation(col_name):
-                ret_lst.append(col_name)
+                if hasattr(self.obj, col_name):
+                    if (not hasattr(getattr(self.obj, col_name), '__call__') or 
+                        hasattr(getattr(self.obj, col_name), '_col_name')):
+                        ret_lst.append(col_name)
+                else:
+                     ret_lst.append(col_name)
         return ret_lst
 
     def get_file_column_list(self):
@@ -371,7 +477,6 @@ class SQLAInterface(BaseInterface):
     def get_image_column_list(self):
         return [i.name for i in self.obj.__mapper__.columns if isinstance(i.type, ImageColumn)]
 
-
     def get_property_first_col(self, col_name):
         # support for only one col for pk and fk
         return self.list_properties[col_name].columns[0]
@@ -383,20 +488,11 @@ class SQLAInterface(BaseInterface):
     def get(self, id):
         return self.session.query(self.obj).get(id)
 
-
-    """
-    ----------- GET PK NAME -------------------
-    """
     def get_pk_name(self):
         for col_name in self.list_columns.keys():
             if self.is_pk(col_name):
                 return col_name
 
-    def get_pk_value(self, item):
-        for col_name in self.list_columns.keys():
-            if self.is_pk(col_name):
-                return getattr(item, col_name)
-
 """
     For Retro-Compatibility
 """
