@@ -6,8 +6,9 @@ import random
 import datetime
 from sqlalchemy import Column, Integer, String, ForeignKey, Date, Float
 from sqlalchemy.orm import relationship
+from flask import request, session
 from flask.ext.appbuilder import Model, SQLA
-from flask_appbuilder.models.filters import FilterStartsWith, FilterEqual
+from flask_appbuilder.models.sqla.filters import FilterStartsWith, FilterEqual
 from flask_appbuilder.models.mixins import FileColumn, ImageColumn
 from flask_appbuilder.views import MasterDetailView, CompactCRUDMixin
 from flask_appbuilder.charts.views import (ChartView, TimeChartView,
@@ -58,6 +59,8 @@ class Model2(Model):
     field_integer = Column(Integer())
     field_float = Column(Float())
     field_date = Column(Date())
+    excluded_string = Column(String(50), default='EXCLUDED')
+    default_string = Column(String(50), default='DEFAULT')
     group_id = Column(Integer, ForeignKey('model1.id'), nullable=False)
     group = relationship("Model1")
 
@@ -67,11 +70,12 @@ class Model2(Model):
     def field_method(self):
        return "field_method_value"                                               
 
+
 class FlaskTestCase(unittest.TestCase):
     def setUp(self):
         from flask import Flask
         from flask.ext.appbuilder import AppBuilder
-        from flask.ext.appbuilder.models.datamodel import SQLAModel
+        from flask.ext.appbuilder.models.sqla.interface import SQLAInterface
         from flask.ext.appbuilder.views import ModelView
 
         self.app = Flask(__name__)
@@ -94,54 +98,52 @@ class FlaskTestCase(unittest.TestCase):
 
 
         class Model2View(ModelView):
-            datamodel = SQLAModel(Model2)
-            list_columns = ['field_integer', 'field_float', 'field_string', 'field_method']
-
-            edit_form_query_rel_fields = [('group',
-                                   SQLAModel(Model1, self.db.session),
-                                   [['field_string', FilterEqual, 'G2']]
-                                  )
-                                ]
-
-            add_form_query_rel_fields = [('group',
-                                   SQLAModel(Model1, self.db.session),
-                                   [['field_string', FilterEqual, 'G1']]
-                                  )
-                                ]
+            datamodel = SQLAInterface(Model2)
+            list_columns = ['field_integer', 'field_float', 'field_string', 'field_method', 'group.field_string']
+            edit_form_query_rel_fields = {'group':[['field_string', FilterEqual, 'G2']]}
+            add_form_query_rel_fields = {'group':[['field_string', FilterEqual, 'G1']]}
+            
+        class Model22View(ModelView):
+            datamodel = SQLAInterface(Model2)
+            list_columns = ['field_integer', 'field_float', 'field_string', 'field_method', 'group.field_string']
+            add_exclude_columns = ['excluded_string']
+            edit_exclude_columns = ['excluded_string']
+            show_exclude_columns = ['excluded_string']
 
 
         class Model1View(ModelView):
-            datamodel = SQLAModel(Model1)
+            datamodel = SQLAInterface(Model1)
             related_views = [Model2View]
             list_columns = ['field_string','field_file']
 
+
         class Model1CompactView(CompactCRUDMixin, ModelView):
-            datamodel = SQLAModel(Model1)
+            datamodel = SQLAInterface(Model1)
 
 
         class Model1Filtered1View(ModelView):
-            datamodel = SQLAModel(Model1)
+            datamodel = SQLAInterface(Model1)
             base_filters = [['field_string', FilterStartsWith, 'a']]
 
 
         class Model1MasterView(MasterDetailView):
-            datamodel = SQLAModel(Model1)
+            datamodel = SQLAInterface(Model1)
             related_views = [Model2View]
 
 
         class Model1Filtered2View(ModelView):
-            datamodel = SQLAModel(Model1)
+            datamodel = SQLAInterface(Model1)
             base_filters = [['field_integer', FilterEqual, 0]]
 
 
         class Model2ChartView(ChartView):
-            datamodel = SQLAModel(Model2)
+            datamodel = SQLAInterface(Model2)
             chart_title = 'Test Model1 Chart'
             group_by_columns = ['field_string']
 
 
         class Model2GroupByChartView(GroupByChartView):
-            datamodel = SQLAModel(Model2)
+            datamodel = SQLAInterface(Model2)
             chart_title = 'Test Model1 Chart'
 
             definitions = [
@@ -155,7 +157,7 @@ class FlaskTestCase(unittest.TestCase):
             ]
             
         class Model2DirectByChartView(DirectByChartView):
-            datamodel = SQLAModel(Model2)
+            datamodel = SQLAInterface(Model2)
             chart_title = 'Test Model1 Chart'
 
             definitions = [
@@ -166,21 +168,21 @@ class FlaskTestCase(unittest.TestCase):
             ]
 
         class Model2TimeChartView(TimeChartView):
-            datamodel = SQLAModel(Model2)
+            datamodel = SQLAInterface(Model2)
             chart_title = 'Test Model1 Chart'
             group_by_columns = ['field_date']
 
         class Model2DirectChartView(DirectChartView):
-            datamodel = SQLAModel(Model2)
+            datamodel = SQLAInterface(Model2)
             chart_title = 'Test Model1 Chart'
             direct_columns = {'stat1': ('group', 'field_integer')}
 
         class Model1MasterView(MasterDetailView):
-            datamodel = SQLAModel(Model1)
+            datamodel = SQLAInterface(Model1)
             related_views = [Model2View]
 
         class Model1MasterChartView(MasterDetailView):
-            datamodel = SQLAModel(Model1)
+            datamodel = SQLAInterface(Model1)
             related_views = [Model2DirectByChartView]
 
 
@@ -192,6 +194,7 @@ class FlaskTestCase(unittest.TestCase):
         self.appbuilder.add_view(Model1Filtered2View, "Model1Filtered2", category='Model1')
 
         self.appbuilder.add_view(Model2View, "Model2")
+        self.appbuilder.add_view(Model22View, "Model22")
         self.appbuilder.add_view(Model2View, "Model2 Add", href='/model2view/add')
         self.appbuilder.add_view(Model2ChartView, "Model2 Chart")
         self.appbuilder.add_view(Model2GroupByChartView, "Model2 Group By Chart")
@@ -200,7 +203,8 @@ class FlaskTestCase(unittest.TestCase):
         self.appbuilder.add_view(Model2DirectChartView, "Model2 Direct Chart")
 
         self.appbuilder.add_view(PSView, "Generic DS PS View", category='PSView')
-
+        role_admin = self.appbuilder.sm.find_role('Admin')
+        self.appbuilder.sm.add_user('admin','admin','user','admin@fab.org',role_admin,'general')
 
     def tearDown(self):
         self.appbuilder = None
@@ -259,8 +263,19 @@ class FlaskTestCase(unittest.TestCase):
         """
             Test views creation and registration
         """
-        eq_(len(self.appbuilder.baseviews), 24)  # current minimal views are 11
+        eq_(len(self.appbuilder.baseviews), 26)  # current minimal views are 11
         
+    def test_back(self):
+        """
+            Test Back functionality
+        """
+        with self.app.test_client() as c:
+            self.login(c, DEFAULT_ADMIN_USER, DEFAULT_ADMIN_PASSWORD)
+            rv = c.get('/model1view/list/?_flt_0_field_string=f')
+            rv = c.get('/model2view/list/')
+            rv = c.get('/back', follow_redirects=True)
+            assert request.args['_flt_0_field_string'] == u'f'
+            assert '/model1view/list/' == request.path
 
     def test_model_creation(self):
         """
@@ -393,6 +408,38 @@ class FlaskTestCase(unittest.TestCase):
         model = self.db.session.query(Model1).first()
         eq_(model, None)
 
+    def test_excluded_cols(self):
+        """
+            Test add_exclude_columns, edit_exclude_columns, show_exclude_columns
+        """
+        client = self.app.test_client()
+        rv = self.login(client, DEFAULT_ADMIN_USER, DEFAULT_ADMIN_PASSWORD)
+        rv = client.get('/model22view/add')
+        eq_(rv.status_code, 200)
+        data = rv.data.decode('utf-8')
+        ok_('field_string' in data)
+        ok_('field_integer' in data)
+        ok_('field_float' in data)
+        ok_('field_date' in data)
+        ok_('excluded_string' not in data)
+        self.insert_data2()
+        rv = client.get('/model22view/edit/1')
+        eq_(rv.status_code, 200)
+        data = rv.data.decode('utf-8')
+        ok_('field_string' in data)
+        ok_('field_integer' in data)
+        ok_('field_float' in data)
+        ok_('field_date' in data)
+        ok_('excluded_string' not in data)
+        rv = client.get('/model22view/show/1')
+        eq_(rv.status_code, 200)
+        data = rv.data.decode('utf-8')
+        ok_('Field String' in data)
+        ok_('Field Integer' in data)
+        ok_('Field Float' in data)
+        ok_('Field Date' in data)
+        ok_('Excluded String' not in data)
+        
 
     def test_query_rel_fields(self):
         """
@@ -403,7 +450,7 @@ class FlaskTestCase(unittest.TestCase):
         self.insert_data2()
 
         # Base filter string starts with
-        rv = client.get('/model2view/add?next=%2Fmodel2view%2Flist%2F')
+        rv = client.get('/model2view/add')
         data = rv.data.decode('utf-8')
         ok_('G1' in data)
         ok_('G2' not in data)
@@ -525,7 +572,6 @@ class FlaskTestCase(unittest.TestCase):
         data = rv.data.decode('utf-8')
         ok_('field_method_value' in data)
 
-
     def test_compactCRUDMixin(self):
         """
             Test CompactCRUD Mixin view
@@ -536,7 +582,6 @@ class FlaskTestCase(unittest.TestCase):
         rv = client.get('/model1compactview/list/')
         eq_(rv.status_code, 200)
 
-
     def test_charts_view(self):
         """
             Test Various Chart views
