@@ -11,18 +11,23 @@ from ._compat import as_unicode
 
 log = logging.getLogger(__name__)
 
-class RenderTemplateWidget(object):
 
+class RenderTemplateWidget(object):
+    """
+        Base template for every widget
+        Enables the possibility of rendering a template
+         inside a template with run time options
+    """
     template = 'appbuilder/general/widgets/render.html'
     template_args = None
 
     def __init__(self, **kwargs):
         self.template_args = kwargs
-    
+
     def __call__(self, **kwargs):
         ctx = _request_ctx_stack.top
         jinja_env = ctx.app.jinja_env
-        
+
         template = jinja_env.get_template(self.template)
         args = self.template_args.copy()
         args.update(kwargs)
@@ -40,14 +45,51 @@ class FormWidget(RenderTemplateWidget):
     """
     template = 'appbuilder/general/widgets/form.html'
 
+
+class FormVerticalWidget(RenderTemplateWidget):
+    """
+        FormWidget
+
+        form = None
+        include_cols = []
+        exclude_cols = []
+        fieldsets = []
+    """
+    template = 'appbuilder/general/widgets/form_vertical.html'
+
+
+class FormHorizontalWidget(RenderTemplateWidget):
+    """
+        FormWidget
+
+        form = None
+        include_cols = []
+        exclude_cols = []
+        fieldsets = []
+    """
+    template = 'appbuilder/general/widgets/form_horizontal.html'
+
+
+class FormInlineWidget(RenderTemplateWidget):
+    """
+        FormWidget
+
+        form = None
+        include_cols = []
+        exclude_cols = []
+        fieldsets = []
+    """
+    template = 'appbuilder/general/widgets/form_inline.html'
+
+
 class GroupFormListWidget(RenderTemplateWidget):
-    template = 'appbuilder/general/widgets/group_form_list.html'    
+    template = 'appbuilder/general/widgets/group_form_list.html'
 
 
 class SearchWidget(FormWidget):
     template = 'appbuilder/general/widgets/search.html'
     filters = None
-    
+
     def __init__(self, **kwargs):
         self.filters = kwargs.get('filters')
         return super(SearchWidget, self).__init__(**kwargs)
@@ -75,7 +117,7 @@ class SearchWidget(FormWidget):
 
 class ShowWidget(RenderTemplateWidget):
     """
-        ShowWidget implements an template as an widget
+        ShowWidget implements a template as an widget
         it takes the following arguments
 
         pk = None
@@ -88,10 +130,15 @@ class ShowWidget(RenderTemplateWidget):
     """
     template = 'appbuilder/general/widgets/show.html'
 
+
 class ShowBlockWidget(RenderTemplateWidget):
     template = 'appbuilder/general/widgets/show_block.html'
 
 
+class ShowVerticalWidget(RenderTemplateWidget):
+    template = 'appbuilder/general/widgets/show_vertical.html'
+
+
 class ListWidget(RenderTemplateWidget):
     """
         List Widget implements a Template as an widget.
@@ -110,7 +157,7 @@ class ListWidget(RenderTemplateWidget):
         modelview_name = ''
     """
     template = 'appbuilder/general/widgets/list.html'
-    
+
 
 class ListMasterWidget(ListWidget):
     template = 'appbuilder/general/widgets/list_master.html'
@@ -125,11 +172,15 @@ class ListAddWidget(ListWidget):
     def __call__(self, **kwargs):
         return super(ListAddWidget, self).__call__(**kwargs)
 
-        
+
 class ListThumbnail(ListWidget):
     template = 'appbuilder/general/widgets/list_thumbnail.html'
 
 
+class ListLinkWidget(ListWidget):
+    template = 'appbuilder/general/widgets/list_link.html'
+
+
 class ListCarousel(ListWidget):
     template = 'appbuilder/general/widgets/list_carousel.html'
 
